# frozen_string_literal: true

require 'cht-chaphql'
require 'cht-chaphql/graphql_formatters'

RESERVATION_RECOMMENDATIONS_FMTRS =
  YAML.load_file(File.join(__dir__, '../helpers/formatters.yml'))

EXTERNAL_SERVICE_FMTRS =
    YAML.load_file(File.join(__dir__, '../helpers/external_service_formatters.yml'))

# module for reservation-recommendations
module ChtApitestReservationRecommendations
  include Chaphql
  include Chaphql::GraphQLHelpers

  # module for your shortcuts
  module ReservationRecommendationsHelpers
    # insertion point for shortcuts

    def node(args = {})
      args[:service] = ReservationRecommendationsService.new
      sym = first_no_cap(camelize('node')).to_sym
      headers = apitest_headers(
          args.merge(with_auth: true,
                     auth_service: args[:auth_service]))
      data = format_request(sym, args)
      variables = {"id"=>args[:id]}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def azure_ri_rec_details_usage(args = {})
      args[:service] = ReservationRecommendationsService.new
      sym = first_no_cap(camelize('azure_ri_rec_details_usage')).to_sym
      headers = apitest_headers(
          args.merge(with_auth: true,
                     auth_service: args[:auth_service]))
      data = format_request(sym, args)
      variables = {"id"=>args[:id]}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def azure_ri_rec_details_info(args = {})
      args[:service] = ReservationRecommendationsService.new
      sym = first_no_cap(camelize('azure_ri_rec_details_info')).to_sym
      headers = apitest_headers(
          args.merge(with_auth: true,
                     auth_service: args[:auth_service]))
      data = format_request(sym, args)
      variables = {"recommendationScope"=>{"scope"=>args[:scope], "scopeLevel"=>args[:scopeLevel], "scopeValues"=>args[:scopeValues]}}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def azure_subscriptions(args = {})
      args[:service] = DataConnectConfigService.new
      sym = first_no_cap(camelize('azure_subscriptions')).to_sym
      headers = apitest_headers(
          args.merge(with_auth: true,
                     auth_service: args[:auth_service]))
      data = format_request(sym, args)
      variables = { }
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def reservation_recommendations(args = {})
      args[:service] = ReservationRecommendationsService.new
      sym = first_no_cap(camelize('reservation_recommendations')).to_sym
      headers = apitest_headers(
          args.merge(with_auth: true,
                     auth_service: args[:auth_service]))
      data = format_request(sym, args)
      variables = {"cloud"=>args[:cloud], "scopeInput"=>{"awsJobId"=>args[:awsJobId]}}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def reservation_recommendations_summary(args = {})
      args[:service] = ReservationRecommendationsService.new
      sym = first_no_cap(camelize('reservation_recommendations_summary')).to_sym
      headers = apitest_headers(
          args.merge(with_auth: true,
                     auth_service: args[:auth_service]))
      data = format_request(sym, args)
      variables = {"cloud"=>args[:cloud], "scopeInput"=>{"awsJobId"=>args[:awsJobId]}}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def refresh_reservation_recommendation_source(args = {})
      args[:service] = ReservationRecommendationsService.new
      sym = first_no_cap(camelize('refresh_reservation_recommendation_source')).to_sym
      headers = apitest_headers(
          args.merge(with_auth: true,
                     auth_service: args[:auth_service]))
      data = format_request(sym, args)
      variables = {"billingAccount"=>args[:billingAccount]}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def update_reservation_recommendations(args = {})
      args[:service] = ReservationRecommendationsService.new
      sym = first_no_cap(camelize('update_reservation_recommendations')).to_sym
      headers = apitest_headers(
          args.merge(with_auth: true,
                     auth_service: args[:auth_service]))
      data = format_request(sym, args)
      variables = {"analysisInput"=>{"awsAnalysisInput"=>args[:awsAnalysisInput]}, "cloud"=>args[:cloud]}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def reservation_recommendation_usage(args = {})
      args[:service] = ReservationRecommendationsService.new
      sym = first_no_cap(camelize('reservation_recommendation_usage')).to_sym
      headers = apitest_headers(
          args.merge(with_auth: true,
                     auth_service: args[:auth_service]))
      data = format_request(sym, args)
      variables = {"recommendationId"=>args[:recommendationId]}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def generate_recommendations_status(args = {})
      args[:service] = ReservationRecommendationsService.new
      sym = first_no_cap(camelize('generate_recommendations_status')).to_sym
      headers = apitest_headers(
          args.merge(with_auth: true,
                     auth_service: args[:auth_service]))
      data = format_request(sym, args)
      variables = {"billingAccount"=>args[:billingAccount]}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args)
      json_body[:data][sym]
    end

    def artifact_job_statuses(args = {})
      args[:service] = SpAllocationSchedulerService.new
      sym = first_no_cap(camelize('artifact_job_statuses')).to_sym
      headers = apitest_headers(
          args.merge(with_auth: true,
                     auth_service: auth_service))
      data = args[:body] || format_request(sym, args)
      variables = {"artifactType"=>args[:artifactType], "customerInfo"=>{"accountId"=>args[:accountId], "customerId"=>args[:customerId], "organizationId"=>args[:organizationId]}}
      graphql url(args), data, variables, headers, args
      expect_success_graphql
      check(sym, args) unless args[:body]
      json_body[:data][sym]
    end

    # end of shortcut insertion
  end

  # class for your service
  class ReservationRecommendationsService < ChtNgService
    include RSpec::Matchers

    RESERVATION_RECOMMENDATIONS_DEFAULTS = {
      name: 'ReservationRecommendations',
      default_namespace: 'reservation-recommendations',
      service_name: 'reservation-recommendations',
      pr_parent: 'reservation-recommendations',
      formatters: RESERVATION_RECOMMENDATIONS_FMTRS['formatters'],
      v2_checkers: RESERVATION_RECOMMENDATIONS_FMTRS['v2_checkers']
    }.freeze
    def initialize(args = {})
      super(args.merge(RESERVATION_RECOMMENDATIONS_DEFAULTS))
    end
  end

  # class for api-proxy support of reservation-recommendations service
  class ApiProxyReservationRecommendations < ChtNgService
    include RSpec::Matchers

    API_PROXY_DEFAULTS = {
      name: 'ApiProxyReservationRecommendations',
      default_namespace: 'api-proxy',
      service_name: 'api-proxy',
      pr_parent: 'api-proxy',
      formatters: RESERVATION_RECOMMENDATIONS_FMTRS['formatters'],
      v2_checkers: RESERVATION_RECOMMENDATIONS_FMTRS['v2_checkers']
    }.freeze
    def initialize(args = {})
      super(args.merge(API_PROXY_DEFAULTS))
    end
  end

  # class for data-config support of reservation-recommendations service
  class DataConnectConfigService < ChtNgService
    include RSpec::Matchers

    DATA_CONNECT_CONFIG_DEFAULTS = {
        name: 'DataConnectConfig',
        default_namespace: 'data-connect-config',
        service_name: 'data-connect-config',
        pr_parent: 'data-connect-config',
        formatters: EXTERNAL_SERVICE_FMTRS['formatters'],
        v2_checkers: EXTERNAL_SERVICE_FMTRS['v2_checkers']
    }.freeze
    def initialize(args = {})
      super(args.merge(DATA_CONNECT_CONFIG_DEFAULTS))
    end
  end

  # class for sp-allocation-scheduler service
  class SpAllocationSchedulerService < ChtNgService
    include RSpec::Matchers

    SP_ALLOCATION_SCHEDULER_DEFAULTS = {
        name: 'SpAllocationScheduler',
        default_namespace: 'sp-allocation-scheduler',
        service_name: 'sp-allocation-scheduler',
        pr_parent: 'sp-allocation-scheduler',
        formatters: EXTERNAL_SERVICE_FMTRS['formatters'],
        v2_checkers: EXTERNAL_SERVICE_FMTRS['v2_checkers']
    }.freeze
    def initialize(args = {})
      super(args.merge(SP_ALLOCATION_SCHEDULER_DEFAULTS))
    end
  end
end
