require 'spec_helper'

module ReservationRecommendationsTest
  describe 'ReservationRecommendations reservationRecommendations' do
    include_context 'request metadata'

    it 'reservationRecommendations request: Last 7 days, all upfront, 1 year, Redshift' do
      pending 'CLHC-219733'
      services = ['Redshift']
      job_id = get_update_job_id('Last7Days', 'AllUpfront', 'OneYear', services)
      wrap_request do
        variables = {
          cloud: 'AWS',
          scopeInput: {
            awsJobId: job_id
          }
        }
        graphql url(service: service),
          format_request(
            :reservationRecommendations,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:reservationRecommendations, service: service)
      end
      aws_recommendations_verification(json_body[:data], 'Last7Days', 1, services, 'AllUpfront')
    end

    it 'reservationRecommendations request: Last 30 days, no upfront, 3 year, ElastiCache' do
      services = ['ElastiCache']
      job_id = get_update_job_id('Last30Days', 'NoUpfront', 'ThreeYear', services)
      wrap_request do
        variables = {
          cloud: 'AWS',
          scopeInput: {
            awsJobId: job_id
          }
        }
        graphql url(service: service),
          format_request(
            :reservationRecommendations,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:reservationRecommendations, service: service)
      end
      aws_recommendations_verification(json_body[:data], 'Last30Days', 3, services, 'NoUpfront')
    end

    it 'reservationRecommendations request: Last 60 days, partial upfront, 3 year, no services included' do
      services = []
      job_id = get_update_job_id('Last60Days', 'PartialUpfront', 'ThreeYear', services)
      wrap_request do
        variables = {
          cloud: 'AWS',
          scopeInput: {
            awsJobId: job_id
          }
        }
        graphql url(service: service),
          format_request(
            :reservationRecommendations,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:reservationRecommendations, service: service)
      end
      aws_recommendations_verification(json_body[:data], 'Last60Days', 3, services, 'PartialUpfront')
    end

    it 'reservationRecommendations request: Last 90 days, all upfront, 3 year, all services included' do
      services = [
        'Redshift',
        'EC2',
        'RDS',
        'ElastiCache',
        'OpenSearch',
        'DynamoDB'
      ]
      job_id = get_update_job_id('Last90Days', 'AllUpfront', 'ThreeYear', services)
      wrap_request do
        variables = {
          cloud: 'AWS',
          scopeInput: {
            awsJobId: job_id
          }
        }
        graphql url(service: service),
          format_request(
            :reservationRecommendations,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:reservationRecommendations, service: service)
      end
      aws_recommendations_verification(json_body[:data], 'Last90Days', 3, services, 'AllUpfront')
    end

    # TODO: uncomment/update when custom request behavior is established
    xit 'reservationRecommendations request: Custom, no upfront, 3 year, EC2' do
      services = ['EC2']
      job_id = get_update_job_id('Custom', 'NoUpfront', 'OneYear', services)
      wrap_request do
        variables = {
          cloud: 'AWS',
          scopeInput: {
            awsJobId: job_id
          }
        }
        graphql url(service: service),
          format_request(
            :reservationRecommendations,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:reservationRecommendations, service: service)
      end
      aws_recommendations_verification(json_body[:data], 'Last60Days', 3, services, 'NoUpfront')
    end
  end
end
