require 'spec_helper'

module ReservationRecommendationsTest
  describe 'ReservationRecommendations reservationRecommendationsSummary' do
    include_context 'request metadata'

    RSpec.shared_examples 'summary tests' do |period, term, payment_type|
      it 'Summary values reflect recommendations values for job id' do
        services = [
            'Redshift',
            'EC2',
            'RDS',
            'ElastiCache',
            'OpenSearch',
            'DynamoDB'
        ]
        term_string = term == 1 ? 'OneYear' : 'ThreeYear'
        job_id = get_update_job_id(period, payment_type, term_string, services)
        wrap_request do
          variables = {
            cloud: 'AWS',
            scopeInput: {
              awsJobId: job_id
            }
          }
          graphql url(service: service),
            format_request(
              :reservationRecommendations,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:reservationRecommendations, service: service)
        end
        recommendations_array = json_body[:data][:reservationRecommendations][:edges]
        recommendation_presence_check(recommendations_array, 'AWS')

        expected_monthly_savings = 0
        expected_annual_savings = 0
        expected_upfront_cost = 0
        expected_savings_opportunities = 0
        total_monthly_net_savings = 0.0
        total_annual_net_savings = 0.0
        recommendations_array.each { |rec|
          expected_monthly_savings += rec[:node][:savingsProperties][:monthlySavings]
          expected_annual_savings += rec[:node][:savingsProperties][:annualSavings]
          expected_upfront_cost += rec[:node][:savingsProperties][:upfrontCost]
          expected_savings_opportunities += rec[:node][:savingsProperties][:recommendedQuantity]
          net_savings = rec[:node][:savingsProperties][:savings]
          total_monthly_net_savings += (net_savings / (term * 12))
          total_annual_net_savings += (net_savings / term)
        }

        wrap_request do
          variables = {
            cloud: 'AWS',
            scopeInput: {
              awsJobId: job_id
            }
          }
          graphql url(service: service),
            format_request(
              :reservationRecommendationsSummary,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:reservationRecommendationsSummary, service: service)
        end
        recommendation_summary = json_body[:data][:reservationRecommendationsSummary]
        expect(recommendation_summary[:projectedMonthlySavings]).to be_within(1).of(expected_monthly_savings)
        expect(recommendation_summary[:projectedAnnualSavings]).to be_within(1).of(expected_annual_savings)
        expect(recommendation_summary[:totalUpfrontCost]).to be_within(1).of(expected_upfront_cost)
        expect(recommendation_summary[:totalSavingsOpportunities]).to eq(expected_savings_opportunities)

        expect(recommendation_summary[:projectedMonthlySavings].round(2)).to eq(total_monthly_net_savings.round(2))
        expect(recommendation_summary[:projectedAnnualSavings].round(2)).to eq(total_annual_net_savings.round(2))
      end
    end

    context 'Summary request: Last 30 Days, 3 Year, No Upfront' do
      include_examples 'summary tests', 'Last30Days', 3, 'NoUpfront'
    end

    context 'Summary request: Last 7 Days, 1 Year, All Upfront' do
      include_examples 'summary tests', 'Last7Days', 1, 'AllUpfront'
    end

    context 'Summary request: Last 60 Days, 1 Year, Partial Upfront' do
      include_examples 'summary tests', 'Last60Days', 1, 'PartialUpfront'
    end
  end
end
