require 'spec_helper'

module ReservationRecommendationsTest
  describe 'ReservationRecommendations azureRIRecDetailsUsage' do
    include_context 'request metadata'
    RSpec.shared_examples 'usage tests' do |scope_in_request,scope_level_in_request,scope_values_in_request|
      it 'azureRIRecDetailsUsage returns valid response for all recommendations in scope' do
        variables = {
          recommendationScope: {
            scope: scope_in_request,
            scopeLevel:  scope_level_in_request,
            scopeValues: scope_values_in_request
          }
        }

        # get all recommendations per scope requested
        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendations_array = json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        usage_verification_ran = false
        resource_types_tested = []

        # run usage response verification for every recommendation in azureRIRecDetailsInfo response
        # only runs usage tests once for each product/resource type
        recommendations_array.each {|rec|
          if resource_types_tested.include?(rec[:node][:resource][:product])
            next
          end

          resource_types_tested << rec[:node][:resource][:product]

          details_info_recommendation_node = rec[:node]
          if details_info_recommendation_node[:savings][:lookBackPeriod] == 'Last60Days'
            lookBackPeriod = 60
          elsif details_info_recommendation_node[:savings][:lookBackPeriod] == 'Last30Days'
            lookBackPeriod = 30
          else
            lookBackPeriod = 7
          end

          wrap_request do
            variables = {
              'id': details_info_recommendation_node[:id]
            }
            graphql url(service: service),
              format_request(
                :azureRIRecDetailsUsage,
                service: service
              ),
              variables, headers
            expect_success_graphql
            check(:azureRIRecDetailsUsage, service: service)
          end
          recommendation_usage = json_body[:data][:azureRIRecDetailsUsage]

          consumption_date_range = (Date.parse(recommendation_usage[:lastConsumptionDate]) - Date.parse(recommendation_usage[:firstConsumptionDate])).to_i
          expect(consumption_date_range).to be > 1
          expect(consumption_date_range).to be <= lookBackPeriod

          expect(recommendation_usage[:usageData].length).to be > 0
          expect(recommendation_usage[:usageData].sum).to be > 0.0
          usage_verification_ran = true
        }
        if usage_verification_ran == false
          raise Exception, 'usage tests did not run, verify recommendations are available for request type under test'
        end
      end
    end

    context 'Usage request, scope: Single scope, ENROLLMENT scopeLevel (Default)' do
      include_examples 'usage tests', 'Single', 'ENROLLMENT', [SHARED_SCOPE_IDS[:enrollment]]
    end

    context 'Usage request, scope: Shared scope, ENROLLMENT scopeLevel' do
      include_examples 'usage tests', 'Shared', 'ENROLLMENT', [SHARED_SCOPE_IDS[:enrollment]]
    end

    # unskip or delete below tests on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-47430
    xcontext 'Usage request, scope: Single scope, BILLING_PROFILE scopeLevel (Default)' do
      include_examples 'usage tests', 'Single', 'BILLING_PROFILE', [SHARED_SCOPE_IDS[:billing_profile]]
    end

    # unskip or delete below tests on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-47430
    xcontext 'Usage request, scope: Shared scope, BILLING_PROFILE scopeLevel (Default)' do
      include_examples 'usage tests', 'Shared', 'BILLING_PROFILE', [SHARED_SCOPE_IDS[:billing_profile]]
    end

    # unskip or delete below tests on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-50335
    xcontext 'Usage request, scope: Single scope, SUBSCRIPTION scopeLevel, PAYG' do
      include_examples 'usage tests', 'Single', 'SUBSCRIPTION', PAYG_SINGLE_SUBSCRIPTION_IDS
    end

    it 'returns expected response for request with invalid id' do
      wrap_request do
        variables = {
          'id': 'crn:#{customerId}:azure-reservation-recommendation/okely-dokely'
        }
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsUsage,
            service: service
          ),
          variables, headers
        expect(json_body[:data][:azureRIRecDetailsUsage]).to eq(nil)
        expect(json_body[:errors][0][:extensions][:classification]).to eq('DataFetchingException')
      end
    end
  end
end
