require 'spec_helper'

module ReservationRecommendationsAllSubscriptionsTest
  describe 'ReservationRecommendations azureRIRecDetailsInfo: All subscriptions (default) cases' do
    include_context 'request metadata'

    RSpec.shared_examples 'all subscription tests' do |scope_level_in_request, single_scope_values, shared_scope_values|
      it "Default recommendations response for #{scope_level_in_request}
            scope level contains expected values" do
        # Get all recommendations for current known subscriptions, for scopeLevel in test
        variables = {
          recommendationScope: {
            scope: 'Single',
            scopeLevel: 'SUBSCRIPTION',
            scopeValues: single_scope_values
          }
        }
        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        subscription_level_scope_values = []
        subscription_level_rec_ids = []
        applied_scopes = []
        # get scope values and recommendation id's that will be expected in default/all subscription response
        json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges].each{|recommendation|
          subscription_level_scope_values << recommendation[:node][:scopeValue]
          subscription_level_rec_ids << recommendation[:node][:id]
        }

        variables = {
          recommendationScope: {
            scope: 'Single',
            scopeLevel: scope_level_in_request,
            scopeValues: shared_scope_values
          }
        }
        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendation_array =
          json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(recommendation_array.length).to be > 0
        scope_value_array = []
        recommendation_array.each{|recommendation|
          expect(recommendation[:node][:scope]).to eq('Single')
          expect(recommendation[:node][:scopeLevel]).to eq('SUBSCRIPTION')
          scope_value_array << recommendation[:node][:scopeValue]
          applied_scopes << recommendation[:node][:resource][:appliedScopes]
        }

        azure_recommendations_verification(SHARED_SCOPE_IDS[:enrollment] , variables, json_body)

        expect(recommendation_array.length).to be >= subscription_level_scope_values.length
        recommendation_array.each{|recommendation|
          expect(subscription_level_scope_values).to include(recommendation[:node][:scopeValue])
          expect(subscription_level_rec_ids).to include(recommendation[:node][:id])
        }

        expect(scope_value_array).to_not include(SHARED_SCOPE_IDS[:enrollment])
        scope_value_array.each{|scope_value|
          expect(PAYG_SINGLE_SUBSCRIPTION_IDS).to_not include(scope_value)
          if scope_level_in_request == 'ENROLLMENT'
            expect(MCA_SINGLE_SUBSCRIPTION_IDS).to_not include(scope_value)
          else
            expect(EA_SINGLE_SUBSCRIPTION_IDS).to_not include(scope_value)
          end
        }
      end

      it "Default response for  #{scope_level_in_request} scopeLevel should contain
            recommendations for subscriptions in appliedScope array of shared recommendations  " do
        # Get all applied scopes in Shared recommendations
        variables = {
          recommendationScope: {
            scope: 'Shared',
            scopeLevel: scope_level_in_request,
            scopeValues: shared_scope_values
          }
        }
        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        applied_scopes = []
        json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges].each{|recommendation|
          applied_scopes.concat(recommendation[:node][:resource][:appliedScopes])
        }

        variables = {
          recommendationScope: {
            scope: 'Single',
            scopeLevel: scope_level_in_request,
            scopeValues: shared_scope_values
          }
        }
        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end

        recommendation_array =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(recommendation_array.length).to be > 0
        recommendation_array.each{|default_recommendation|
          expect(applied_scopes).to include(default_recommendation[:node][:scopeValue])
        }
      end
    end

    # CLHC-219732
    xcontext 'Get recommendations for all subscriptions under ENROLLMENT' do
      include_examples 'all subscription tests', 'ENROLLMENT', EA_SINGLE_SUBSCRIPTION_IDS, [SHARED_SCOPE_IDS[:enrollment]]
    end

    # unskip or delete below tests on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-47430
    xcontext 'Get recommendations for all subscriptions under BILLING_PROFILE' do
      include_examples 'all subscription tests', 'BILLING_PROFILE', MCA_SINGLE_SUBSCRIPTION_IDS, [SHARED_SCOPE_IDS[:billing_profile]]
    end
  end
end

