require 'spec_helper'

module ReservationRecommendationsPaginationTest
  RSpec.shared_examples 'standard pagination tests' do |scope, scope_level, scope_values|
    include_context 'request metadata'

    unfiltered_recommendations_array = []
    recommendations_cursor_array = []
    first_and_last_value = 0
    mid_cursor_index =  0
    mid_cursor = ""
    before(:each) {
      if recommendations_cursor_array.empty?
        if scope_values.nil?
          if USE_AZURE_SUBSCRIPTION_API
            all_available_subs = get_all_subscriptions()
            value_with_most_recs = get_value_with_most_quotes('Single', 'SUBSCRIPTION', all_available_subs)
            scope_values = value_with_most_recs
          else
            skip(SKIP_AZURE_SUBSCRIPTION_MSG)
          end
        end
        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values}
        }
        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges].each { |recommendation|
          unfiltered_recommendations_array << recommendation
          recommendations_cursor_array << recommendation[:cursor]
        }
        recommendation_presence_check(recommendations_cursor_array, 'AZURE')
        # the below is intended to ensure page range is always
        # within valid range regardless of test data being used
        first_and_last_value = (recommendations_cursor_array.length / 2).round() - 2
        mid_cursor_index =  recommendations_cursor_array.length/ 2
        mid_cursor = recommendations_cursor_array[mid_cursor_index]
      end
    }

    it 'Pagination: first and after' do
      expected_first_cursor = recommendations_cursor_array[mid_cursor_index + 1]
      expected_end_cursor = recommendations_cursor_array[mid_cursor_index + first_and_last_value]

      variables = {
        recommendationScope: {
          scope: scope,
          scopeLevel:  scope_level,
          scopeValues: scope_values},
        first: first_and_last_value,
        after: mid_cursor
      }

      wrap_request do
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:azureRIRecDetailsInfo, service: service)
      end
      page_info = json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:pageInfo]
      recommendations =
          json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
      expect(recommendations[0][:cursor]).to eq(expected_first_cursor)
      expect(page_info[:startCursor]).to eq(recommendations[0][:cursor])
      expect(page_info[:endCursor]).to eq(expected_end_cursor)
      expect(page_info[:hasNextPage]).to eq(true)
      expect(page_info[:hasPreviousPage]).to eq(true)
      expect(recommendations.length).to eq(first_and_last_value)
      azure_recommendations_verification(scope_values, variables, json_body)
    end

    it 'Pagination: last and before' do
      expected_first_cursor = recommendations_cursor_array[mid_cursor_index - first_and_last_value]
      expected_end_cursor = recommendations_cursor_array[mid_cursor_index - 1]

      variables = {
        recommendationScope: {
          scope: scope,
          scopeLevel:  scope_level,
          scopeValues: scope_values},
        last: first_and_last_value,
        before: mid_cursor
      }

      wrap_request do
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:azureRIRecDetailsInfo, service: service)
      end

      page_info = json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:pageInfo]
      recommendations =
          json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
      expect(recommendations[0][:cursor]).to eq(expected_first_cursor)
      expect(recommendations.length).to eq(first_and_last_value)
      expect(page_info[:startCursor]).to eq(recommendations[0][:cursor])
      expect(page_info[:endCursor]).to eq(expected_end_cursor)
      expect(page_info[:hasNextPage]).to eq(true)
      expect(page_info[:hasPreviousPage]).to eq(true)
      azure_recommendations_verification(scope_values, variables, json_body)
    end

    it 'Pagination: no previous page' do
      variables = {
        recommendationScope: {
          scope: scope,
          scopeLevel:  scope_level,
          scopeValues: scope_values},
        last: 1,
        before: recommendations_cursor_array[1]
      }

      wrap_request do
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:azureRIRecDetailsInfo, service: service)
      end
      page_info = json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:pageInfo]
      expect(page_info[:hasPreviousPage]).to eq(false)
      expect(page_info[:hasNextPage]).to eq(true)
      azure_recommendations_verification(scope_values, variables, json_body)
    end

    it 'Pagination: no next page' do
      second_last_cursor_index =  recommendations_cursor_array.length - 2

      variables = {
        recommendationScope: {
          scope: scope,
          scopeLevel:  scope_level,
          scopeValues: scope_values},
        first: 1,
        after: recommendations_cursor_array[second_last_cursor_index]
      }

      wrap_request do
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:azureRIRecDetailsInfo, service: service)
      end
      page_info = json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:pageInfo]
      expect(page_info[:hasPreviousPage]).to eq(true)
      expect(page_info[:hasNextPage]).to eq(false)
      azure_recommendations_verification(scope_values, variables, json_body)
    end

    # test to verify pagination, filter, and sort, all work when included in same request
    it 'Pagination with sort and filter fields, DESC with last field/value' do
      expected_filtered_quote_number = 0
      unfiltered_recommendations_array.each { |rec|
        if rec[:node][:savings][:reservationOrderTerm] == 'P3Y'
          expected_filtered_quote_number += 1
        end
      }

      # get initial filtered response without any sorting/pagination
      variables = {
        recommendationScope: {
          scope: scope,
          scopeLevel:  scope_level,
          scopeValues: scope_values},
        filterRules: [
          {field: 'reservationOrderTerm',
           filterType: 'EQ',
           filterValues: ['p3y'],
          },
          {field: 'resourceType',
           filterType: 'NE',
           filterValues: ['CosmosDB'],
          }
        ]
      }
      wrap_request do
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:azureRIRecDetailsInfo, service: service)
      end
      rec_cursor_array = []
      json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges].each { |recommendation|
        rec_cursor_array << recommendation[:cursor]
      }
      # the below is intended to ensure page range is always
      # within valid range regardless of test data being used
      first_value = (rec_cursor_array.length / 2).round() - 2
      cursor_index =  rec_cursor_array.length/ 2
      selected_cursor = rec_cursor_array[cursor_index]
      expected_first_cursor = rec_cursor_array[cursor_index + 1]
      expected_end_cursor = rec_cursor_array[cursor_index + first_value]

      variables = {
        recommendationScope: {
          scope: scope,
          scopeLevel:  scope_level,
          scopeValues: scope_values},
        filterRules: [
          {field: 'reservationOrderTerm',
           filterType: 'EQ',
           filterValues: ['p3y']},
          {field: 'resourceType',
           filterType: 'NE',
           filterValues: ['CosmosDB'],
          }
        ],
        sortRules: [
          {field: 'product',
           direction: 'DESC'}
        ],
        first: first_value,
        after: selected_cursor
      }

      wrap_request do
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:azureRIRecDetailsInfo, service: service)
      end
      page_info = json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:pageInfo]
      filtered_recommendations =
          json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
      expect(filtered_recommendations.length).to eq(first_value)

      last_value = filtered_recommendations[0][:node][:resource][:product]
      # all returned quotes should have filtered term and sorting
      filtered_recommendations.each { |rec|
        expect(rec[:node][:savings][:reservationOrderTerm]).to eq('P3Y')
        expect(rec[:node][:resource][:product]).to be <= (last_value)
        last_value = rec[:node][:resource][:product]
      }
      # verify correct pagination
      expect(filtered_recommendations[0][:cursor]).to eq(expected_first_cursor)
      expect(page_info[:startCursor]).to eq(filtered_recommendations[0][:cursor])
      expect(page_info[:endCursor]).to eq(expected_end_cursor)
      expect(page_info[:hasNextPage]).to eq(true)
      expect(page_info[:hasPreviousPage]).to eq(true)
      expect(filtered_recommendations.length).to eq(first_value)
      azure_recommendations_verification(scope_values, variables, json_body)
    end

    # test to verify behavior when after value is beyond range of recommendations available
    it 'Pagination: out of range' do
      expected_first_cursor = recommendations_cursor_array[mid_cursor_index + 1]
      expected_end_cursor = recommendations_cursor_array.last

      # total array length from after mid array should be out of range
      variables = {
        recommendationScope: {
          scope: scope,
          scopeLevel:  scope_level,
          scopeValues: scope_values},
        first: recommendations_cursor_array.length,
        after: mid_cursor
      }

      wrap_request do
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:azureRIRecDetailsInfo, service: service)
      end
      page_info = json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:pageInfo]
      recommendations =
          json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
      expect(recommendations[0][:cursor]).to eq(expected_first_cursor)
      expect(page_info[:startCursor]).to eq(recommendations[0][:cursor])
      expect(page_info[:endCursor]).to eq(expected_end_cursor)
      expect(page_info[:hasNextPage]).to eq(false)
      expect(page_info[:hasPreviousPage]).to eq(true)
      azure_recommendations_verification(scope_values, variables, json_body)
    end

    context 'totalCount tests' do
      test_label = 'total recommendation count should be greater than recommendation number returned'
      before(:each) {
        if unfiltered_recommendations_array.length < 2
          skip('Not enough unfiltered recommendations available to verify totalCount behavior')
        end
      }

      it "#{test_label} when first filter applied" do
        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          first: first_and_last_value,
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        total_count = json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:totalCount]
        recommendations = json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(recommendations.length).to be < total_count
      end

      it "#{test_label} when last filter applied" do
        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          last: first_and_last_value,
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        total_count = json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:totalCount]
        recommendations = json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(recommendations.length).to be < total_count
      end
    end
  end

  describe 'ReservationRecommendations azureRIRecDetailsInfo: pagination tests' do
    include_context 'request metadata'

    context 'Single scope, SUBSCRIPTION scope level, single subscription' do
      include_examples 'standard pagination tests', 'Single', 'SUBSCRIPTION'
    end

    context 'Single scope, SUBSCRIPTION scope level, multiple subscriptions' do
      include_examples 'standard pagination tests', 'Single', 'SUBSCRIPTION', EA_SINGLE_SUBSCRIPTION_IDS
    end

    context 'Shared scope, ENROLLMENT scope level cases' do
      include_examples 'standard pagination tests', 'Shared', 'ENROLLMENT', [SHARED_SCOPE_IDS[:enrollment]]
    end

    # unskip or delete below tests on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-47430
    xcontext 'Shared scope, BILLING_PROFILE scope level cases' do
      include_examples 'standard pagination tests', 'Shared', 'BILLING_PROFILE', [SHARED_SCOPE_IDS[:billing_profile]]
    end

    context 'negative tests' do
      it 'Pagination: invalid cursor value' do
        variables = {
          recommendationScope: {
            scope: 'Shared',
            scopeLevel:  'ENROLLMENT',
            scopeValues: [SHARED_SCOPE_IDS[:enrollment]]},
          first: 5,
          after: 'invalid cursor'
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect(json_body[:errors]).not_to be_empty
          expect(json_body[:data]).to be_nil
        end
      end
    end
  end
end
