require 'spec_helper'

module ReservationRecommendationsTest
  # Disable this test temporally
  # TODO: ENG-49512 (Investigate Azure scope comparison api test failure)
  xdescribe 'Savings at Shared scope should be >= savings at Single scope' do
    include_context 'request metadata'

    RSpec.shared_examples 'Savings scope level comparison' do | scope_level, scope_value, filter_rules |
      it 'Savings at Shared scope should be >= savings at Single scope' do
        variables = {
          recommendationScope: {
            scope: 'Shared',
            scopeLevel: scope_level,
            scopeValues: scope_value
          },
          filterRules: filter_rules
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        shared_summary_monthly_savings = json_body[:data][:azureRIRecDetailsInfo][:projectedMonthlySavings]

        variables = {
          recommendationScope: {
            scope: 'Single',
            scopeLevel: scope_level,
            scopeValues: scope_value
          },
          filterRules: filter_rules
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        single_summary_monthly_savings = json_body[:data][:azureRIRecDetailsInfo][:projectedMonthlySavings]

        expect(single_summary_monthly_savings).to be > 0
        expect(shared_summary_monthly_savings).to be >= single_summary_monthly_savings
      end
    end

    context 'ENROLLMENT scope level savings verification: unfiltered' do
      include_examples 'Savings scope level comparison',
                       'ENROLLMENT',
                       [SHARED_SCOPE_IDS[:enrollment]],
                       []
    end

    context 'ENROLLMENT scope level savings verification: filtered (3 years, last 30 days)' do
      include_examples 'Savings scope level comparison',
                       'ENROLLMENT',
                       [SHARED_SCOPE_IDS[:enrollment]],
                       [{field: 'lookBackPeriod', filterType: 'EQ', filterValues: ['Last30Days']},
                        {field: 'reservationOrderTerm', filterType: 'EQ', filterValues: ['P3Y']}]
    end

    # unskip or delete below tests on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-47430
    xcontext 'BILLING_PROFILE scope level savings verification: unfiltered' do
      include_examples 'Savings scope level comparison',
                       'BILLING_PROFILE',
                       [SHARED_SCOPE_IDS[:billing_profile]],
                       []
    end

    # unskip or delete below tests on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-47430
    xcontext 'BILLING_PROFILE scope level savings verification: filtered (3 years, last 30 days)' do
      include_examples 'Savings scope level comparison',
                       'BILLING_PROFILE',
                       [SHARED_SCOPE_IDS[:billing_profile]],
                       [{field: 'lookBackPeriod', filterType: 'EQ', filterValues: ['Last30Days']},
                        {field: 'reservationOrderTerm', filterType: 'EQ', filterValues: ['P3Y']}]
    end
  end
end
