require 'spec_helper'

module ReservationRecommendationsSortTest
  RSpec.shared_examples 'standard sort tests' do |scope, scope_level, scope_values|
    include_context 'request metadata'

    unsorted_recommendations_array = []
    first_and_last_value = 0
    before(:each) {
      if unsorted_recommendations_array.empty?
        if scope_values.nil?
          if USE_AZURE_SUBSCRIPTION_API
            all_available_subs = get_all_subscriptions()
            value_with_most_recs = get_value_with_most_quotes('Single', 'SUBSCRIPTION', all_available_subs)
            scope_values = value_with_most_recs
          else
            skip(SKIP_AZURE_SUBSCRIPTION_MSG)
          end
        end
        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values}
        }
        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges].each { |recommendation|
          unsorted_recommendations_array << recommendation
        }
        # value to be applied to first and last fields in sorted requests
        # using below assignment to guarantee a valid value less than max quote number is used
        first_and_last_value = (unsorted_recommendations_array.length / 2).round()
      end
    }

    context 'Sort on number field' do
      it 'sort ASC on number (upfrontCost)' do
        cost_array = []
        unsorted_recommendations_array.each { |rec|
          cost_array << rec[:node][:savings][:upfrontCost]
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          sortRules: [
            {field: 'upfrontCost',
             direction: 'ASC'}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendation_array =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(recommendation_array.length).to eq(unsorted_recommendations_array.length)
        expect(recommendation_array[0][:node][:savings][:upfrontCost]).to eq(cost_array.min)
        last_index = recommendation_array.length - 1
        expect(recommendation_array[last_index][:node][:savings][:upfrontCost]).to eq(cost_array.max)
        last_value = recommendation_array[0][:node][:savings][:upfrontCost]
        recommendation_array.each { |rec|
          expect(rec[:node][:savings][:upfrontCost]).to be >= (last_value)
          last_value = rec[:node][:savings][:upfrontCost]
        }
        azure_recommendations_verification(scope_values, variables, json_body)
      end

      it 'sort DESC on number (monthlyPayment)' do
        monthly_payment_array = []
        unsorted_recommendations_array.each { |rec|
          monthly_payment_array << rec[:node][:savings][:monthlyPayment]
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          sortRules: [
            {field: 'monthlyPayment',
             direction: 'DESC'}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendation_array =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(recommendation_array.length).to eq(unsorted_recommendations_array.length)
        expect(recommendation_array[0][:node][:savings][:monthlyPayment]).to eq(monthly_payment_array.max)
        last_value = recommendation_array[0][:node][:savings][:monthlyPayment]
        recommendation_array.each { |rec|
          expect(rec[:node][:savings][:monthlyPayment]).to be <= (last_value)
          last_value = rec[:node][:savings][:monthlyPayment]
        }
        azure_recommendations_verification(scope_values, variables, json_body)
      end

      it 'sort ASC with first field/value, on number (savings)' do
        savings_array = []
        unsorted_recommendations_array.each { |rec|
          savings_array << rec[:node][:savings][:savings]
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          sortRules: [
            {field: 'savings',
             direction: 'ASC'}
          ],
          first: first_and_last_value
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendation_array =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        last_value = recommendation_array[0][:node][:savings][:savings]
        expect(recommendation_array[0][:node][:savings][:savings]).to eq(savings_array.min)
        sorted_cost_array = []
        recommendation_array.each { |rec|
          expect(rec[:node][:savings][:savings]).to be >= (last_value)
          last_value = rec[:node][:savings][:savings]
          sorted_cost_array << last_value
        }
        expect(sorted_cost_array).to include(savings_array.min)
        expect(recommendation_array.length).to eq first_and_last_value
        azure_recommendations_verification(scope_values, variables, json_body)
      end

      it 'sort DESC with first field/value, on number (monthlySavings)' do
        monthly_savings_array = []
        unsorted_recommendations_array.each { |rec|
          monthly_savings_array << rec[:node][:savings][:monthlySavings]
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          sortRules: [
            {field: 'monthlySavings',
             direction: 'DESC'}
          ],
          first: first_and_last_value
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendation_array =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        last_value = recommendation_array[0][:node][:savings][:monthlySavings]
        expect(recommendation_array[0][:node][:savings][:monthlySavings]).to eq(monthly_savings_array.max)
        sorted_cost_array = []
        recommendation_array.each { |rec|
          expect(rec[:node][:savings][:monthlySavings]).to be <= (last_value)
          last_value = rec[:node][:savings][:monthlySavings]
          sorted_cost_array << last_value
        }
        expect(sorted_cost_array).to include(monthly_savings_array.max)
        expect(recommendation_array.length).to eq first_and_last_value
        azure_recommendations_verification(scope_values, variables, json_body)
      end

      it 'sort ASC with last field/value, on number (annualSavings)' do
        annual_savings_array = []
        unsorted_recommendations_array.each { |rec|
          annual_savings_array << rec[:node][:savings][:annualSavings]
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          sortRules: [
            {field: 'annualSavings',
             direction: 'ASC'}
          ],
          last: first_and_last_value
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendation_array =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        last_index = recommendation_array.length - 1
        expect(recommendation_array[last_index][:node][:savings][:annualSavings]).to eq(annual_savings_array.max)
        last_value = recommendation_array[0][:node][:savings][:annualSavings]
        sorted_cost_array = []
        recommendation_array.each { |rec|
          expect(rec[:node][:savings][:annualSavings]).to be >= (last_value)
          last_value = rec[:node][:savings][:annualSavings]
          sorted_cost_array << last_value
        }
        expect(sorted_cost_array).to include(annual_savings_array.max)
        expect(recommendation_array.length).to eq first_and_last_value
        azure_recommendations_verification(scope_values, variables, json_body)
      end

      it 'sort on number field  with string field filter' do
        annual_savings_array = []
        unsorted_recommendations_array.each { |rec|
          if rec[:node][:savings][:reservationOrderTerm] == 'P3Y'
            annual_savings_array << rec[:node][:savings][:annualSavings]
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'reservationOrderTerm',
             filterType: 'EQ',
             filterValues: ['p3y']}
          ],
          sortRules: [
            {field: 'annualSavings',
             direction: 'ASC'}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendation_array =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(recommendation_array.length).to eq(annual_savings_array.length)
        last_value = recommendation_array[0][:node][:savings][:annualSavings]
        expect(recommendation_array[0][:node][:savings][:annualSavings]).to eq(annual_savings_array.min)
        recommendation_array.each { |rec|
          expect(rec[:node][:savings][:reservationOrderTerm]).to eq('P3Y')
          expect(rec[:node][:savings][:annualSavings]).to be >= (last_value)
          last_value = rec[:node][:savings][:annualSavings]
        }
        azure_recommendations_verification(scope_values, variables, json_body)
      end

      it 'sort on number with filter on same field (annualSavings)' do
        annual_savings_array = []
        annual_savings_total = 0
        unsorted_recommendations_array.each { |rec|
          annual_savings_total += rec[:node][:savings][:annualSavings]
        }
        mean_savings = annual_savings_total / unsorted_recommendations_array.length.to_f

        unsorted_recommendations_array.each { |rec|
          if rec[:node][:savings][:annualSavings] > mean_savings
            annual_savings_array << rec[:node][:savings][:annualSavings]
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'annualSavings',
             filterType: 'GT',
             filterValues: [mean_savings]}
          ],
          sortRules: [
            {field: 'annualSavings',
             direction: 'ASC'}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendation_array =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(recommendation_array.length).to eq(annual_savings_array.length)
        expect(recommendation_array[0][:node][:savings][:annualSavings]).to be > mean_savings
        last_value = recommendation_array[0][:node][:savings][:annualSavings]
        recommendation_array.each { |rec|
          expect(rec[:node][:savings][:annualSavings]).to be >= (last_value)
          last_value = rec[:node][:savings][:annualSavings]
        }
        azure_recommendations_verification(scope_values, variables, json_body)
      end
    end

    context 'Sort on string field' do
      it 'sort ASC on string (reservationOrderTerm)' do
        term_array = []
        unsorted_recommendations_array.each { |rec|
          term_array << rec[:node][:savings][:reservationOrderTerm]
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          sortRules: [
            {field: 'reservationOrderTerm',
             direction: 'ASC'}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendation_array =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(recommendation_array.length).to eq(unsorted_recommendations_array.length)
        expect(recommendation_array[0][:node][:savings][:reservationOrderTerm]).to eq(term_array.min)
        last_value = recommendation_array[0][:node][:savings][:reservationOrderTerm]
        recommendation_array.each { |rec|
          expect(rec[:node][:savings][:reservationOrderTerm].downcase).to be >= (last_value.downcase)
          last_value = rec[:node][:savings][:reservationOrderTerm]
        }
        azure_recommendations_verification(scope_values, variables, json_body)
      end

      it 'sort DESC on string (product)' do
        product_array = []
        unsorted_recommendations_array.each { |rec|
          product_array << rec[:node][:resource][:product]
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          sortRules: [
            {field: 'product',
             direction: 'DESC'}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendation_array =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(recommendation_array.length).to eq(unsorted_recommendations_array.length)
        expect(recommendation_array[0][:node][:resource][:product]).to eq(product_array.max)
        last_value = recommendation_array[0][:node][:resource][:product]
        recommendation_array.each { |rec|
          expect(rec[:node][:resource][:product].downcase).to be <= (last_value.downcase)
          last_value = rec[:node][:resource][:product]
        }
        azure_recommendations_verification(scope_values, variables, json_body)
      end

      it 'sort ASC with first field/value, on string (product)' do
        product_array = []
        unsorted_recommendations_array.each { |rec|
          product_array << rec[:node][:resource][:product]
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          sortRules: [
            {field: 'product',
             direction: 'ASC'}
          ],
          first: first_and_last_value
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendation_array =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        last_value = recommendation_array[0][:node][:resource][:product]
        expect(recommendation_array[0][:node][:resource][:product]).to eq(product_array.min)
        sorted_product_array = []
        recommendation_array.each { |rec|
          expect(rec[:node][:resource][:product].downcase).to be >= (last_value.downcase)
          last_value = rec[:node][:resource][:product]
          sorted_product_array << last_value
        }
        expect(sorted_product_array).to include(product_array.min)
        expect(recommendation_array.length).to eq first_and_last_value
        azure_recommendations_verification(scope_values, variables, json_body)
      end

      it 'sort and filter on same string field (lookBackPeriod)' do
        look_back_period_array = []
        unsorted_recommendations_array.each { |rec|
          if rec[:node][:savings][:lookBackPeriod] != 'Last7Days'
            look_back_period_array << rec[:node][:savings][:lookBackPeriod]
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'lookBackPeriod',
             filterType: 'NE',
             filterValues: ['Last7Days']}
          ],
          sortRules: [
            {field: 'lookBackPeriod',
             direction: 'DESC'}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendation_array =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(recommendation_array.length).to eq(look_back_period_array.length)
        last_value = recommendation_array[0][:node][:savings][:lookBackPeriod]
        expect(recommendation_array[0][:node][:savings][:lookBackPeriod]).to eq(look_back_period_array.max)
        recommendation_array.each { |rec|
          expect(rec[:node][:savings][:lookBackPeriod]).to_not eq('Last7Days')
          expect(rec[:node][:savings][:lookBackPeriod].downcase).to be <= (last_value.downcase)
          last_value = rec[:node][:savings][:lookBackPeriod]
        }
        azure_recommendations_verification(scope_values, variables, json_body)
      end

      it 'sort on string field with filter on number field, DESC with last field/value' do
        sku_name_array = []
        cost_total = 0
        unsorted_recommendations_array.each { |rec|
          cost_total = cost_total + rec[:node][:savings][:upfrontCost]
        }
        mean_costs = cost_total / unsorted_recommendations_array.length.to_f

        unsorted_recommendations_array.each { |rec|
          if rec[:node][:savings][:upfrontCost] > mean_costs
            sku_name_array << rec[:node][:resource][:product]
          end
        }
        last_field_value = (sku_name_array.length / 2).round()

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'upfrontCost',
             filterType: 'GT',
             filterValues: [mean_costs]}
          ],
          sortRules: [
            {field: 'product',
             direction: 'DESC'}
          ],
          last: last_field_value
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendation_array =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(recommendation_array.length).to eq(last_field_value)
        last_index = recommendation_array.length - 1
        expect(recommendation_array[last_index][:node][:resource][:product]).to eq(sku_name_array.min)
        last_value = recommendation_array[0][:node][:resource][:product]
        recommendation_array.each { |rec|
          expect(rec[:node][:resource][:product].downcase).to be <= (last_value.downcase)
          last_value = rec[:node][:resource][:product]
        }
        azure_recommendations_verification(scope_values, variables, json_body)
      end

      it 'sort on scopeValue (ASC)' do
        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          sortRules: [
            {field: 'scopeValue',
             direction: 'ASC'}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendation_array =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        last_value = recommendation_array[0][:node][:scopeValue]
        recommendation_array.each { |rec|
          expect(rec[:node][:scopeValue]).to be >= (last_value)
          last_value = rec[:node][:scopeValue]
        }
        azure_recommendations_verification(scope_values, variables, json_body)
      end

      it 'sort on scopeName (ASC)' do
        if USE_AZURE_SUBSCRIPTION_API
          if scope == 'Shared'
            skip('scopeName is null in shared scope')
          end
          # request to get account subscription info for data name/value consistency check
          wrap_request do
            variables = {
            }
            graphql url(service: data_service),
              format_request(
                :azureSubscriptions,
                service: data_service
              ),
              variables, headers
            expect_success_graphql
            check(:azureSubscriptions, service: data_service)
          end
          azure_subscription_array = json_body[:data][:azureSubscriptions][:edges]

          variables = {
            recommendationScope: {
              scope: scope,
              scopeLevel:  scope_level,
              scopeValues: scope_values},
            sortRules: [
              {field: 'scopeName',
               direction: 'ASC'}
            ]
          }

          wrap_request do
            graphql url(service: service),
              format_request(
                :azureRIRecDetailsInfo,
                service: service
              ),
              variables, headers
            expect_success_graphql
            check(:azureRIRecDetailsInfo, service: service)
          end
          recommendation_array =
              json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
          last_value = recommendation_array[0][:node][:scopeName]
          recommendation_array.each { |rec|
            expect(rec[:node][:scopeName].downcase).to be >= (last_value.downcase)

            # ensure scopeName and ScopeValue stay consistent post sort
            azure_subscription_array.each {|subscription|
              if subscription[:node][:name] == rec[:node][:scopeName]
                expect(rec[:node][:scopeValue]).to eq(subscription[:node][:azureId])
                break
              end
            }
            last_value = rec[:node][:scopeName]
          }
          azure_recommendations_verification(scope_values, variables, json_body)
        else
          skip(SKIP_AZURE_SUBSCRIPTION_MSG)
        end
      end
    end

    context 'Multiple sort rules' do
      it 'sort gets applied to first rule when applicable' do
        cost_array = []
        unsorted_recommendations_array.each { |rec|
          cost_array << rec[:node][:savings][:upfrontCost]
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          sortRules: [
            {field: 'upfrontCost',
             direction: 'ASC'},
            {field: 'savings',
             direction: 'DESC'}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendation_array =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(recommendation_array.length).to eq(unsorted_recommendations_array.length)
        expect(recommendation_array[0][:node][:savings][:upfrontCost]).to eq(cost_array.min)
        last_index = recommendation_array.length - 1
        expect(recommendation_array[last_index][:node][:savings][:upfrontCost]).to eq(cost_array.max)
        last_value = recommendation_array[0][:node][:savings][:upfrontCost]
        recommendation_array.each { |rec|
          expect(rec[:node][:savings][:upfrontCost]).to be >= (last_value)
          last_value = rec[:node][:savings][:upfrontCost]
        }
        azure_recommendations_verification(scope_values, variables, json_body)
      end

      it 'sort gets applied to second rule when first not applicable' do
        cost_array = []
        unsorted_recommendations_array.each { |rec|
          if rec[:node][:savings][:lookBackPeriod] == 'Last30Days'
            cost_array << rec[:node][:savings][:upfrontCost]
          end
        }

        variables = {
          recommendationScope: {
            scope: scope,
            scopeLevel:  scope_level,
            scopeValues: scope_values},
          filterRules: [
            {field: 'lookBackPeriod',
             filterType: 'EQ',
             filterValues: ['Last30Days']}
          ],
          sortRules: [
            {field: 'lookBackPeriod',
             direction: 'DESC'},
            {field: 'upfrontCost',
             direction: 'ASC'}
          ]
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        recommendation_array =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
        expect(recommendation_array.length).to eq(cost_array.length)
        expect(recommendation_array[0][:node][:savings][:upfrontCost]).to eq(cost_array.min)
        last_index = recommendation_array.length - 1
        expect(recommendation_array[last_index][:node][:savings][:upfrontCost]).to eq(cost_array.max)
        last_value = recommendation_array[0][:node][:savings][:upfrontCost]
        recommendation_array.each { |rec|
          expect(rec[:node][:savings][:upfrontCost]).to be >= (last_value)
          last_value = rec[:node][:savings][:upfrontCost]
        }
        azure_recommendations_verification(scope_values, variables, json_body)
      end
    end
  end


  describe 'ReservationRecommendations azureRIRecDetailsInfo: sort tests' do
    context 'Single scope, SUBSCRIPTION scope level, single subscription' do
      include_examples 'standard sort tests', 'Single', 'SUBSCRIPTION'
    end

    context 'Single scope, ENROLLMENT scope level, all subscriptions (default)' do
      include_examples 'standard sort tests', 'Single', 'ENROLLMENT', [SHARED_SCOPE_IDS[:enrollment]]
    end

    context 'Shared scope, ENROLLMENT scope level cases' do
      include_examples 'standard sort tests', 'Shared', 'ENROLLMENT', [SHARED_SCOPE_IDS[:enrollment]]
    end

    # unskip or delete below tests on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-47430
    xcontext 'Shared scope, BILLING_PROFILE scope level cases' do
      include_examples 'standard sort tests', 'Shared', 'BILLING_PROFILE', [SHARED_SCOPE_IDS[:billing_profile]]
    end

  end
end
