require 'spec_helper'

module ReservationRecommendationsTest
  describe 'ReservationRecommendations azureRIRecDetailsInfo: positive cases' do
    include_context 'request metadata'

    RSpec.shared_examples 'recommendation response tests' do |scope_in_request,scope_level_in_request,scope_values_in_request|
      it "request returns expected values for #{scope_in_request} scope, #{scope_level_in_request} scopeLevel, single scope value" do
        multiple_value = false

        if scope_values_in_request.length > 1
          multiple_value = true
        end

        variables = {
          recommendationScope: {
            scope: scope_in_request,
            scopeLevel: scope_level_in_request,
            scopeValues: scope_values_in_request
          }
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        scope_values_in_response = []
        json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges].each { |recommendation|
          scope_values_in_response << recommendation[:node][:scopeValue]
        }
        if multiple_value
          expect(scope_values_in_response.uniq.count).to be > 1
          # verify response contains quotes for both values included in request
          scope_values_in_response.each {|input_value|
            expect(scope_values_in_request).to include(input_value)
          }
        else
          expect(scope_values_in_response.uniq.count).to eq(1)
        end
        azure_recommendations_verification(scope_values_in_request, variables, json_body)
      end
    end

    # unskip or delete below tests on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-50335
    xcontext 'Get quote: Single scope, SUBSCRIPTION scopeLevel, PAYG Subscription id' do
      scope_values = []
      before(:each) {
        scope_values << get_value_with_most_quotes('Single', 'SUBSCRIPTION', PAYG_SINGLE_SUBSCRIPTION_IDS)
      }
      include_examples 'recommendation response tests', 'Single', 'SUBSCRIPTION', scope_values
    end

    context 'Get quote: Single scope, SUBSCRIPTION scopeLevel, EA Subscription id' do
      scope_values = []
      before(:each) {
        scope_values << get_value_with_most_quotes('Single', 'SUBSCRIPTION', EA_SINGLE_SUBSCRIPTION_IDS)
      }
      include_examples 'recommendation response tests', 'Single', 'SUBSCRIPTION', scope_values
    end

    xcontext 'Get quote: Single scope, SUBSCRIPTION scopeLevel, MCA Subscription id' do
      scope_values = []
      before(:each) {
        scope_values << get_value_with_most_quotes('Single', 'SUBSCRIPTION', MCA_SINGLE_SUBSCRIPTION_IDS)
      }
      include_examples 'recommendation response tests', 'Single', 'SUBSCRIPTION', scope_values
    end

    context 'Get quote: Shared scope, ENROLLMENT scopeLevel, single scope value' do
      include_examples 'recommendation response tests', 'Shared', 'ENROLLMENT', [SHARED_SCOPE_IDS[:enrollment]]
    end

    # unskip or delete below test on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-47430
    xcontext 'Shared scope, BILLING_PROFILE scope level cases' do
      include_examples 'recommendation response tests', 'Shared', 'BILLING_PROFILE', [SHARED_SCOPE_IDS[:billing_profile]]
    end

    context 'Get quote: Single scope, SUBSCRIPTION scopeLevel, multiple scope values' do
      all_available_subscriptions = []
      before(:each) {
        if USE_AZURE_SUBSCRIPTION_API
          all_available_subscriptions.concat(get_all_subscriptions())
        else
          skip(SKIP_AZURE_SUBSCRIPTION_MSG)
        end
      }
      include_examples 'recommendation response tests', 'Single', 'SUBSCRIPTION', all_available_subscriptions
    end

    # unskip or delete below tests on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-50335
    xcontext 'Get quote: Single scope, SUBSCRIPTION scopeLevel, PAYG , EA, and MCA ids' do
      include_examples 'recommendation response tests', 'Single', 'SUBSCRIPTION', EA_SINGLE_SUBSCRIPTION_IDS + PAYG_SINGLE_SUBSCRIPTION_IDS + MCA_SINGLE_SUBSCRIPTION_IDS
    end

    # Skipping below tests until these integration tests run in a post deploy stage
    # Currently, these tests run on branch builds of the service, so there's no utility in running
    # these tests via the api-proy, as the api-proxy would route requests to the deployed version
    # of the service built off master, rather than the version of the service built off
    # the branch under test, and running in the branch pr namespace
    xit 'Expected response returned via api-proxy' do
      wrap_request do
        variables = {
          recommendationScope: {
            scope: 'Shared',
            scopeLevel: 'ENROLLMENT',
            scopeValues: SHARED_SCOPE_IDS[:enrollment]
          }
        }
        graphql url(service: ApiProxyReservationRecommendations.new),
          format_request(
            :azureRIRecDetailsInfo,
            service: ApiProxyReservationRecommendations.new
          ),
          variables, headers
        expect_success_graphql
        check(:azureRIRecDetailsInfo, service: ApiProxyReservationRecommendations.new)
        azure_recommendations_verification(SHARED_SCOPE_IDS[:enrollment], variables, json_body)
      end
    end
  end

  # the following tests use azureSubscriptions api to validate consistency in azureReservationRecommendations
  describe 'Azure subscription consistency' do
    include_context 'request metadata'
    azure_subscription_array = []

    before(:each) {
      if USE_AZURE_SUBSCRIPTION_API
        if azure_subscription_array.empty?
          wrap_request do
            variables = {
            }
            graphql url(service: data_service),
              format_request(
                :azureSubscriptions,
                service: data_service
              ),
              variables, headers
            expect_success_graphql
            check(:azureSubscriptions, service: data_service)
          end
          azure_subscription_array = json_body[:data][:azureSubscriptions][:edges]
          if azure_subscription_array.empty?
            raise Exception.new "There are no azure subscriptions available for redshirt"
          end
        end
      else
        skip(SKIP_AZURE_SUBSCRIPTION_MSG)
      end

    }

    it 'Valid response returned for all redshirt subscriptions, available via azureSubscriptions' do
      tests_ran = false
      azure_subscription_array.each { |subscription|
        variables = {
          recommendationScope: {
            scope: 'Single',
            scopeLevel: 'SUBSCRIPTION',
            scopeValues: subscription[:node][:azureId]
          }
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
          recommendation_response  = json_body[:data][:azureRIRecDetailsInfo]
          recommendation_array = json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
          if recommendation_array.length > 0
            azure_recommendations_verification(subscription[:node][:azureId], variables, json_body)
            # verify that each scope value is asscoiated with the expected scope name
            recommendation_array.each {|recommendation|
              expect(recommendation[:node][:scopeName]).to eq(subscription[:node][:name])
              expect(recommendation[:node][:scopeValue]).to eq(subscription[:node][:azureId])
            }
          else
            expect(recommendation_response[:totalUpfrontCost]).to eq(0.round())
            expect(recommendation_response[:totalSavingsOpportunities]).to eq(0.round())
            expect(recommendation_response[:projectedMonthlySavings]).to eq(0.round())
            expect(recommendation_response[:projectedMonthlySavings]).to eq(0.round())
            expect(recommendation_array.length).to eq(0)
          end
        end
        tests_ran = true
      }
      if !tests_ran
        raise Exception.new "Tests did not run, verify subscriptions are available"
      end
    end


    it 'All recommendations returned for default ENROLLMENT request are for redshirt subscriptions' do
      expected_subscriptions = []
      recommendation_array = []
      azure_subscription_array.each { |subscription|
        expected_subscriptions << subscription[:node][:azureId]
      }

      variables = {
        recommendationScope: {
          scope: 'Single',
          scopeLevel: 'ENROLLMENT',
          scopeValues: [SHARED_SCOPE_IDS[:enrollment]]
        }
      }

      wrap_request do
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:azureRIRecDetailsInfo, service: service)
        recommendation_array =
          json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
      end
      if recommendation_array.length == 0
        raise Exception, "No recommendations available for redshirt under ENROLLMENT scope"
      end
      recommendation_array.each { |rec|
        expect(expected_subscriptions).to include(rec[:node][:scopeValue])
      }
    end

    # unskip or delete below test on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-47430
    xit 'All recommendations returned for default BILLING_PROFILE request are for redshirt subscriptions' do
      expected_subscriptions = []
      recommendation_array = []
      azure_subscription_array.each { |subscription|
        expected_subscriptions << subscription[:node][:azureId]
      }

      variables = {
        recommendationScope: {
          scope: 'Single',
          scopeLevel: 'BILLING_PROFILE',
          scopeValues: [SHARED_SCOPE_IDS[:billing_profile]]
        }
      }

      wrap_request do
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_success_graphql
        check(:azureRIRecDetailsInfo, service: service)
        recommendation_array =
          json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges]
      end
      if recommendation_array.length == 0
        raise Exception, "No recommendations available for redshirt under billing_profile scope"
      end
      recommendation_array.each { |rec|
        expect(expected_subscriptions).to include(rec[:node][:scopeValue])
      }
    end
  end

  describe 'ReservationRecommendations azureReservationRecommendations: Negative cases' do
    include_context 'request metadata'

    it 'Invalid scope' do
      wrap_request do
        variables = {
          recommendationScope: {
            scope: 'Linked',
            scopeLevel: 'SUBSCRIPTION',
            scopeValues: EA_SINGLE_SUBSCRIPTION_IDS
          }
        }
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:errors][0][:extensions][:classification]).to eq("ValidationError")
        expect(
            json_body[:errors][0][:message]).to include(
              "Invalid input for Enum 'AzureReservationScope'. No value found for name 'Linked'")
      end
    end

    it 'Empty scope value array' do
      wrap_request do
        variables = {
          recommendationScope: {
            scope: 'Single',
            scopeLevel: 'SUBSCRIPTION',
            scopeValues: []
          }
        }
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:errors][0][:extensions][:classification]).to eq("DataFetchingException")
        expect(json_body[:errors][0][:message]).to include("scopeValues can't be empty")
        expect(json_body[:data]).to eq(nil)
      end
    end

    it 'Invalid scopeLevel (lowercase)' do
      wrap_request do
        variables = {
          recommendationScope: {
            scope: 'Single',
            scopeLevel: 'subscription',
            scopeValues: EA_SINGLE_SUBSCRIPTION_IDS
          }
        }
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:errors][0][:extensions][:classification]).to eq("ValidationError")
        expect(
            json_body[:errors][0][:message]).to include(
              "Invalid input for Enum 'AzureReservationScopeLevel'. No value found for name 'subscription'")
      end
    end

    it 'Incorrect scope for scopeLevel specified' do
      wrap_request do
        variables = {
          recommendationScope: {
            scope: 'Shared',
            scopeLevel: 'SUBSCRIPTION',
            scopeValues: [SHARED_SCOPE_IDS[:enrollment]]
          }
        }
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:errors][0][:extensions][:classification]).to eq("DataFetchingException")
        expect(
          json_body[:errors][0][:message]).to include(
                                                "Wrong scope. For subscription scope level, the scope should be Single. For enrollment or billing profile scope level, the scope should be Shared")
      end
    end

    it 'Subscription id can not be used in Shared/ENROLLMENT scope' do
      wrap_request do
        variables = {
          recommendationScope: {
            scope: 'Shared',
            scopeLevel: 'ENROLLMENT',
            scopeValues: PAYG_SINGLE_SUBSCRIPTION_IDS
          }
        }
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:errors][0][:extensions][:classification]).to eq("DataFetchingException")
        expect(
          json_body[:errors][0][:message]).to include(
                                                "Azure scope(subscription, enrollment or billing profile) is either not accessible, or does not exist within your organization")
      end
    end

    it 'Subscription id can not be used in Shared/BILLING_PROFILE scope' do
      wrap_request do
        variables = {
          recommendationScope: {
            scope: 'Shared',
            scopeLevel: 'BILLING_PROFILE',
            scopeValues: PAYG_SINGLE_SUBSCRIPTION_IDS
          }
        }
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:errors][0][:extensions][:classification]).to eq("DataFetchingException")
        expect(
          json_body[:errors][0][:message]).to include(
                                                "Azure scope(subscription, enrollment or billing profile) is either not accessible, or does not exist within your organization")
      end
    end

    it 'ENROLLMENT id can not be used in single/subscription scope' do
      wrap_request do
        variables = {
          recommendationScope: {
            scope: 'Single',
            scopeLevel: 'SUBSCRIPTION',
            scopeValues: [SHARED_SCOPE_IDS[:enrollment]]
          }
        }
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:errors][0][:extensions][:classification]).to eq("DataFetchingException")
        expect(
          json_body[:errors][0][:message]).to include(
                                                "Azure scope(subscription, enrollment or billing profile) is either not accessible, or does not exist within your organization")
      end
    end

    it 'ENROLLMENT id can not be used with BILLING_PROFILE scopeLevel' do
      wrap_request do
        variables = {
          recommendationScope: {
            scope: 'Shared',
            scopeLevel: 'BILLING_PROFILE',
            scopeValues: [SHARED_SCOPE_IDS[:enrollment]]
          }
        }
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, headers
        expect_status(200)
        expect(json_body[:errors][0][:extensions][:classification]).to eq("DataFetchingException")
        expect(
          json_body[:errors][0][:message]).to include(
                                                "Azure scope(subscription, enrollment or billing profile) is either not accessible, or does not exist within your organization")
      end
    end

    # TODO: Update test to expect 401 if ENG-37000 addressed
    it 'returns expected error for no auth request' do
      wrap_request do
        variables = {
          'after': 'after',
          'before': 'before',
          'filterRules': [
            'filterRules'
          ],
          'first': 'first',
          'last': 'last',
          'recommendationScope': {
            scope: 'Single',
            scopeLevel: 'SUBSCRIPTION',
            scopeValues: ["anything"]
          },
          'search': 'search',
          'sortRules': [
            'sortRules'
          ]
        }
        graphql url(service: service),
          format_request(
            :azureRIRecDetailsInfo,
            service: service
          ),
          variables, apitest_headers
        expect_status(403)
      end
    end
  end
end
