require 'spec_helper'

module ReservationRecommendationsTest
  describe 'ReservationRecommendations node' do
    include_context 'request metadata'
    RSpec.shared_examples 'node consistency tests' do |scope_in_request,scope_level_in_request,scope_values_in_request|
      it 'node request response is consistent with recommendation node in azureRIRecDetailsInfo response' do
        variables = {
          recommendationScope: {
            scope: scope_in_request,
            scopeLevel:  scope_level_in_request,
            scopeValues: scope_values_in_request
          }
        }

        wrap_request do
          graphql url(service: service),
            format_request(
              :azureRIRecDetailsInfo,
              service: service
            ),
            variables, headers
          expect_success_graphql
          check(:azureRIRecDetailsInfo, service: service)
        end
        details_info_recommendation_node =
            json_body[:data][:azureRIRecDetailsInfo][:recommendationsDetails][:edges][0][:node]

        wrap_request do
          variables = {}
          graphql url(service: service),
            format_request(
              :node,
              service: service,
              crn: details_info_recommendation_node[:id],
              type: 'AzureRIRecDetailsModel',
              fields: 'id,
                      scope,
                      scopeLevel,
                      scopeValue,
                      scopeName,
                      currency,
                      resource {
                        appliedScopes,
                        onDemandRate,
                        product,
                        region,
                        reservationRate,
                        resourceType,
                      },
                      savings {
                        lookBackPeriod,
                        recommendedQuantity,
                        unitOfMeasure,
                        upfrontCost,
                        monthlyPayment,
                        savings,
                        monthlySavings,
                        annualSavings,
                        reservationOrderTerm,
                        savingsType,
                      }'
            ),
            variables, headers
          expect_success_graphql
          check(:node, service: service)
        end
        node_response = json_body[:data][:node]
        expect(node_response[:scopeLevel]).to eq(details_info_recommendation_node[:scopeLevel])
        expect(node_response[:scopeValue]).to eq(details_info_recommendation_node[:scopeValue])
        expect(node_response[:scopeName]).to eq(details_info_recommendation_node[:scopeName])
        expect(node_response[:currency]).to eq(details_info_recommendation_node[:currency])
        expect(node_response[:resource][:appliedScopes]).to eq(details_info_recommendation_node[:resource][:appliedScopes])
        expect(node_response[:resource][:onDemandRate]).to eq(details_info_recommendation_node[:resource][:onDemandRate])
        expect(node_response[:resource][:product]).to eq(details_info_recommendation_node[:resource][:product])
        expect(node_response[:resource][:region]).to eq(details_info_recommendation_node[:resource][:region])
        expect(node_response[:resource][:reservationRate]).to eq(details_info_recommendation_node[:resource][:reservationRate])
        expect(node_response[:resource][:resourceType]).to eq(details_info_recommendation_node[:resource][:resourceType])
        expect(node_response[:savings][:lookBackPeriod]).to eq(details_info_recommendation_node[:savings][:lookBackPeriod])
        expect(node_response[:savings][:recommendedQuantity]).to eq(details_info_recommendation_node[:savings][:recommendedQuantity])
        expect(node_response[:savings][:unitOfMeasure]).to eq(details_info_recommendation_node[:savings][:unitOfMeasure])
        expect(node_response[:savings][:upfrontCost]).to eq(details_info_recommendation_node[:savings][:upfrontCost])
        expect(node_response[:savings][:monthlyPayment]).to eq(details_info_recommendation_node[:savings][:monthlyPayment])
        expect(node_response[:savings][:savings]).to eq(details_info_recommendation_node[:savings][:savings])
        expect(node_response[:savings][:monthlySavings]).to eq(details_info_recommendation_node[:savings][:monthlySavings])
        expect(node_response[:savings][:annualSavings]).to eq(details_info_recommendation_node[:savings][:annualSavings])
        expect(node_response[:savings][:reservationOrderTerm]).to eq(details_info_recommendation_node[:savings][:reservationOrderTerm])
        expect(node_response[:savings][:savingsType]).to eq(details_info_recommendation_node[:savings][:savingsType])
      end
    end

    context 'Node request: Single scope, ENROLLMENT scopeLevel (Default)' do
      include_examples 'node consistency tests', 'Single', 'ENROLLMENT', [SHARED_SCOPE_IDS[:enrollment]]
    end

    context 'Node request: Shared scope, ENROLLMENT scopeLevel' do
      include_examples 'node consistency tests', 'Shared', 'ENROLLMENT', [SHARED_SCOPE_IDS[:enrollment]]
    end

    # unskip or delete below tests on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-47430
    xcontext 'Node request: Single scope, BILLING_PROFILE scopeLevel (Default)' do
      include_examples 'node consistency tests', 'Single', 'BILLING_PROFILE', [SHARED_SCOPE_IDS[:billing_profile]]
    end

    # unskip or delete below tests on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-47430
    xcontext 'Node request: Shared scope, BILLING_PROFILE scopeLevel (Default)' do
      include_examples 'node consistency tests', 'Shared', 'BILLING_PROFILE', [SHARED_SCOPE_IDS[:billing_profile]]
    end

    # unskip or delete below tests on resolution of https://cloudhealthtech.atlassian.net/browse/ENG-50330
    xcontext 'Node request: Single scope, SUBSCRIPTION scopeLevel, PAYG subscription' do
      include_examples 'node consistency tests', 'Single', 'SUBSCRIPTION', PAYG_SINGLE_SUBSCRIPTION_IDS
    end

    it 'returns expected response for request with invalid id' do
      wrap_request do
        variables = {
          'id': "crn:#{customerId}:azure-reservation-recommendation/okely-dokely"
        }
        graphql url(service: service),
          format_request(
            :node,
            service: service,
            crn: "crn:#{customerId}:azure-reservation-recommendation/okely-dokely",
            type: 'AzureRIRecDetailsModel',
            fields: 'id'
          ),
          variables, headers
        expect(json_body[:data][:azureRIRecDetailsUsage]).to eq(nil)
        expect(json_body[:errors][0][:extensions][:classification]).to eq('DataFetchingException')
      end
    end
  end
end
