# frozen_string_literal: true

require 'spec_helper'

xdescribe 'ReservationRecommendations' do
  include_context 'request metadata'

  it 'can use rspec and get an access token' do
    if getenv == 'local'
      pending 'Default test only runs in dev, '\
              'staging and prod environments'
    end
    expect { tokens[:access_token] }.to_not raise_error
  end

  it 'can find your service in cht-chaphql, introspect and format' do
    expect { url(service: service) }
      .to_not raise_error,
              'Ensure that your service has support added by '\
              'running bin/add_service'
    expect do
      introspect(service: service, with_auth: true,
                 auth_service: auth_service)
    end.to_not raise_error,
               "Introspect for 'reservation-recommendations' failed"
    expect do
      x = analyze(service: service)
      `mkdir supported_schemas &> /dev/null`
      File.write('supported_schemas/analysis.txt', x)
    end.to_not raise_error
  end

  xit 'can introspect your service and staging api-proxy', staging_env: true do
    introspect(service: service, with_auth: true,
               auth_service: auth_service)
    svc_analysis = analyze(service: service)
    proxy_service = ChtApitestReservationRecommendations::ApiProxyReservationRecommendations.new
    tmp = ENV['TEST_ENV']
    ENV['TEST_ENV'] = 'staging'
    introspect(service: proxy_service, with_auth: true,
               auth_service: auth_service)
    stg_analysis = analyze(service: proxy_service)
    ENV['TEST_ENV'] = tmp
    res = conflict?(svc_analysis, stg_analysis)
    unless res == ''
      puts 'if this service is not exposed through api-proxy feel free to delete the test'
      expect(res)
        .to eq('')
    end
  end

  describe 'introspection - graphql schema changes', not_via_proxy: true do
    include_context 'request metadata'
    wrap_request do
      it 'detects schema changes for reservation-recommendations' do
        introspect(service: service, auth_service: auth_service)
        filename = "supported_schemas/#{service.filename}"
        if File.exist?(filename)
          data = json_body[:data][:__schema]
          x = schema_changed?(filename, data)
          expect(x).to eq([]), <<~OOPS
            The GraphQL schema had an unexpected change.  Please alert the QE or development team
            member responsible for the API tests and make sure they are aware of the schema change.
            If this change is anything other than adding a new back-end service, there may need to
            be new tests added to support the modified schema.

            The apitest/README.md file in this repository has more details about how to avoid this
            error.
            Differences:
            #{x}
          OOPS
        else
          puts "couldn't find #{filename} - skipping test"
        end
      end
    end
  end

  describe 'save schema changes for reservation-recommendations',
           save_schema: true do
    include_context 'request metadata'
    it 'saves schema changes for reservation-recommendations' do
      wrap_request do
        x = introspect(service: service,
                       auth_service: auth_service)
        filename = "supported_schemas/#{service.filename}"
        write_schema(filename, x)
      end
    end
  end
end
